# encoding=CP1251
import os, nltk, math, sys
import cPickle
import random
import re
import string
import numpy
#import mlpy

learn_default_path = r'open\data'
load_default_path = r'models/model'
test_default_path = r'open\data'
test_answers_path = r'open\res'
right_answers_path = r'open\answers'
pathToModel = "models/model"
learn_matrix_file = r'learn_matrix_new.txt'
learn_label_file = r'learn_label_new.txt'
temp_file = r'temp.txt'
another_temp_file = r'another_temp.txt'
idf_abstract_file = r'models/idf_abstract.pkl'
idf_other_file = r'models/idf_other.pkl'
file_counter_file = r'models/file_counter.pkl'
weights_file = r'models/weights.pkl'
default_abstract_length = 300

num_keywords = 7
#number_unigrams = int(sys.argv[1])
#number_bigrams = int(sys.argv[2])

#number_unigrams = 2
#number_bigrams = 4

def del_duplicates(arr_list):
	res = []
	for item in arr_list:
		if not item in res:
			res.append(item)
	return res
	
class PhraseSample():
	def __init__(self, phrase, document, features, label):
		self.obj = dict()
		self.obj['phrase'] = phrase
		self.obj['document'] = document
		self.obj['features'] = features
		self.obj['label'] = label
		
	def get(self):
		return self.obj

class TextProcessor():
	def __init__(self):
		self.abstract_words = []
		self.other_words = []
		self.words = []
		self.original_word_forms = dict()
		self.phrases = []
		
	def process_document(self, document_name):
		document = open(document_name)
		text = unicode(document.read(), "utf-8")
		document.close()
		text = re.sub('\d', '', text)
		text = re.sub('[,\.\!\?\(\)\:\;\'\"\-\[\]\<\>\@\#\$\%\^\&\*\+\=]', ' ', text)
		tokenized_text = nltk.PunktWordTokenizer().tokenize(text)
		stemmer = nltk.stem.snowball.RussianStemmer() 
		words = map(stemmer.stem, tokenized_text)
		text_split = self.get_logical_struct(words)
		self.words = words
		self.abstract_words = text_split[0]
		self.other_words = text_split[1]
		self.save_original_word_forms(words, tokenized_text)
		self.phrases = self.create_phrases(self.abstract_words, document_name)
	
	def process_text(self, text):
		text = re.sub('\d', '', text)
		text = re.sub('[,\.\!\?\(\)\:\;\'\"\-\[\]\<\>\@\#\$\%\^\&\*\+\=]', ' ', text)
		tokenized_text = nltk.PunktWordTokenizer().tokenize(text)
		stemmer = nltk.stem.snowball.RussianStemmer() 
		words = map(stemmer.stem, tokenized_text)
		text_split = self.get_logical_struct(words)
		self.words = words
		self.abstract_words = text_split[0]
		self.other_words = text_split[1]
		self.save_original_word_forms(words, tokenized_text)
		self.phrases = self.create_phrases(self.abstract_words)
	
	def create_phrases(self, words, document_name = None):
		unigrams = set(words)
		unigrams = filter(self.filter_unigram, unigrams)
		unigrams = [[item] for item in unigrams]
		
		bigrams = zip(words[:-1], words[1:])
		bigrams = filter(self.filter_bigram, bigrams)
		bigrams = del_duplicates(bigrams)
		bigrams = [list(item) for item in bigrams]
		
		trigrams = zip(words[:-2], words[1:-1], words[2:])
		trigrams = filter(self.filter_trigram, trigrams)
		trigrams = del_duplicates(trigrams)
		trigrams = [list(item) for item in trigrams]
		
		ngrams = []
		ngrams.extend(unigrams)
		ngrams.extend(bigrams)
		ngrams.extend(trigrams)
		
		res = [PhraseSample(item, document_name, [], 0).get() for item in ngrams]
		return res
		
	def filter_unigram(self, unigram):
		if len(unigram) < 3:
			return False
		else:
			return True
	
	def filter_bigram(self, bigram):
		if len(bigram[0]) < 3 or len(bigram[1]) < 3:
			return False
		else:
			return True
			
	def filter_trigram(self, trigram):
		if len(trigram[0]) < 3 or len(trigram[2]) < 3:
			return False
		else:
			return True
		
	def get_logical_struct(self, stemmed_text):
		key_word = u''
		for i in range(0, len(stemmed_text)):
			if stemmed_text[i] == key_word:
				res = list()
				if i > 300:
					res.append(stemmed_text[0:200])
					res.append(stemmed_text[200:])
				else:
					res.append(stemmed_text[0:i])
					res.append(stemmed_text[i:])
				return res
		res = list()
		res.append(stemmed_text[:default_abstract_length])
		res.append(stemmed_text[default_abstract_length:])
		return res
		
	def save_original_word_forms(self, words, tokenized_text):            
		for i in range(0, len(words)):
			if not(words[i] in self.original_word_forms):
				self.original_word_forms[words[i]] = tokenized_text[i]
    
	def restore_word(self, word):
		return self.original_word_forms[word]
	
	def get_abstract(self):
		return self.abstract_words
		
	def get_other(self):
		return self.other_words
	
	def get_phrases(self):
		return self.phrases
	
	def get_words(self):
		return self.words
		
	def clear(self):
		self.abstract_words = []
		self.other_words = []
		self.words = []
		self.phrases = []
	
	def test(self):
		self.process_document(r'open\data\f_edut-arpg6papehz.txt')
		if os.path.exists(temp_file):
			os.remove(temp_file)
		temp = open(temp_file, "a")	
		for i in range(0, len(self.abstract_words)):
			temp.write(self.abstract_words[i].encode('utf-8') + '\n')
		temp.write('-----------------------------------------------------------------------------------------\n')
		for i in range(0, len(self.other_words)):
			temp.write(self.other_words[i].encode('utf-8') + '\n')
		temp.write('-----------------------------------------------------------------------------------------\n')
		for (key, item) in sorted(self.original_word_forms.items()):
			temp.write(key.encode('utf-8') + ' : ' + item.encode('utf-8') + '\n')
		temp.write('-----------------------------------------------------------------------------------------\n')
		for i in range(0, len(self.phrases)):
			temp.write('Phrase = ')
			for j in range(0, len(self.phrases[i]['phrase'])):
				temp.write(self.phrases[i]['phrase'][j].encode('utf-8') + ' ')
			temp.write(', Document = ' + self.phrases[i]['document'] + '\n')
		temp.close()
			
class Learner():

	def __init__(self):
		self.model = None	
		self.idf_abstract = None
		self.idf_other = None
		self.learn_matrix = []
		self.learn_label = []
		self.weights = []
	
	def learn(self, sample_learn):
		self.compute_idf_all(sample_learn)
		#self.load_idf()
		#self.save_idf()
		self.create_learn(sample_learn)
		self.learn_model()
		#self.save_matrix()
		
	def save_matrix(self):
		numpy.savetxt(learn_matrix_file, self.learn_matrix, delimiter=",", fmt = "%.4f")
		numpy.savetxt(learn_label_file, self.learn_label, delimiter=",", fmt = "%.0f")
		
	def predict(self, text):
		text_processor = TextProcessor()
		text_processor.process_text(text)
		
		abstract = text_processor.get_abstract()
		other = text_processor.get_other()
		words = text_processor.get_words()
		phrases = text_processor.get_phrases()
		self.create_features(phrases, abstract, other, words)
		
		test_matrix = []
		for phrase in phrases:
			test_matrix.append(phrase['features'])
		self.normalize_matrix(test_matrix)
		answers = self.pred_values(test_matrix, self.weights)
		result = []
		for i in range(0, len(answers)):
			result.append([answers[i], i])
		result = sorted(result)
		#numpy.savetxt(learn_matrix_file, test_matrix, delimiter=",", fmt = "%.3f")
		numpy.savetxt(temp_file, result, delimiter=",", fmt = "%.3f")
		key_words = []
		key_words = self.create_res(result, phrases, num_keywords)
		#for i in range(0, min(num_keywords, len(result))):
		#	key_words.append(' '.join(map(text_processor.restore_word, phrases[result[i][1]]['phrase'])))
		key_words = [' '.join(map(text_processor.restore_word, phrase)) for phrase in key_words]
		return key_words
	
	def create_res(self, result, phrases, num_keywords):
		num_keyphrases = 0
		offset = 0
		res_words = []
		while num_keyphrases < num_keywords and offset < len(result):
			for j in range(num_keyphrases, num_keywords):
				res_words.append(phrases[result[offset][1]]['phrase'])
				offset += 1
				num_keyphrases += 1
				if offset == len(result):
					break
			del_range = set()
			for j in range(0, num_keywords):
				if len(res_words[j]) == 1:
					for k in range(0, num_keywords):
						if (not (k == j)) and len(res_words[k]) > 1:
							try: 
								res_words[k].index(res_words[j][0])
							except:
								continue
							del_range.add(j)
							break
			for j in sorted(del_range, reverse = True):
				del res_words[j]
			num_keyphrases = len(res_words)
		return res_words
		
	def pred_values(self, matrix, weights):
		#print (len(matrix[0]), len(weights))
		result = 1.0 / (1 + numpy.exp(-numpy.dot(matrix, weights))) 
		return result
		
	def compute_idf_all(self, sample_learn):
		self.file_counter = 0
		docs_abstract = dict()
		docs_other = dict()
		for file_learn in sample_learn:
			print (file_learn, self.file_counter)
			self.file_counter += 1
			text_processor = TextProcessor()
			text_processor.process_document(file_learn)
			
			abstract = text_processor.get_abstract()
			other = text_processor.get_other()
			phrases = text_processor.get_phrases()
			
			tokens = [item['phrase'] for item in phrases]
			
			self.compute_phrases_count(docs_abstract, abstract, tokens)
			self.compute_phrases_count(docs_other, other, tokens)
		
		self.idf_abstract = dict()
		self.idf_other = dict()
		
		for ngram in docs_abstract.keys():
			if docs_abstract[ngram] > 1:
				self.idf_abstract[ngram] = -math.log(docs_abstract[ngram] / self.file_counter)
		
		for ngram in docs_other.keys():
			if docs_other[ngram] > 1:
				self.idf_other[ngram] = -math.log(docs_other[ngram] / self.file_counter)
	
	def compute_phrases_count(self, docs, text, tokens):
		for token in tokens:
			token_str = ' '.join(token)
			if not token_str in docs.keys():
				docs[token_str] = 0.0
			len_token = len(token)
			for i in range(0, len(text) - len_token + 1):
				if text[i:i+len_token] == token:
					docs[token_str] += 1.0
					break
	
	def create_learn(self, sample_learn):
		num_files = 1
		self.num_find = 0
		self.num_not_find = 0
		for file_learn in sample_learn:
			print (file_learn, num_files)
			num_files += 1
			text_processor = TextProcessor()
			text_processor.process_document(file_learn)
			
			abstract = text_processor.get_abstract()
			other = text_processor.get_other()
			phrases = text_processor.get_phrases()
			words = text_processor.get_words()
			
			self.set_labels(file_learn, phrases)
			self.create_features(phrases, abstract, other, words)
			self.add_learn_matrix(phrases)
			
	def set_labels(self, filename, phrases):
		file_right = open(filename.replace('data', 'answers'))
		text_right = unicode(file_right.read(), "utf-8")
		file_right.close()
		text_right = self.stem(text_right)
		num_find = 0
		res_find = []
		for i in range(0, len(text_right)):
			res_find.append(0)
		for i in range(0, len(phrases)):
			if phrases[i]['phrase'] in text_right:
				phrases[i]['label'] = 1
				num_find += 1
				res_find[text_right.index(phrases[i]['phrase'])] = 1
		#for i in range(0, len(res_find)):
		#	if res_find[i] == 0:
		#		phrases.append(PhraseSample(text_right[i], filename, [], 1).get())	
		self.num_find += num_find
		self.num_not_find += len(text_right) - num_find
		
		
	def stem(self, text):
		stemmer = nltk.stem.snowball.RussianStemmer() 
		text_stem = []
		text = text.split('\n')
		for line in text:
			if line:
				line_free = nltk.PunktWordTokenizer().tokenize(line)
				line_stem = []
				for word in line_free:
					line_stem.append(stemmer.stem(word))
				text_stem.append(line_stem)
		return text_stem 

	def create_features(self, phrases, abstract, other, words): 
		for i in range(0, len(phrases)):
			token = phrases[i]['phrase']
			len_token = len(token)
			tf_abstract = float(self.count_text_token(abstract, token)) / (len(abstract) - len_token + 1)
			tf_other = float(self.count_text_token(other, token)) / (len(other) - len_token + 1)
			token_str = ' '.join(token)
			
			if token_str in self.idf_abstract:
				tf_idf_abstract = float(self.idf_abstract[token_str]) * tf_abstract
			else: 
				tf_idf_abstract = -math.log(1.0 / self.file_counter) * tf_abstract
			if token_str in self.idf_other:
				tf_idf_other = float(self.idf_other[token_str]) * tf_other
			else: 
				tf_idf_other = -math.log(1.0 / self.file_counter) * tf_other
			
			(first_occur, last_occur) = self.find_occur(words, token)
			#(first_occur_abstract, last_occur_abstract) = self.find_occur(abstract, token)
			#(first_occur_other, last_occur_other) = self.find_occur(other, token)
			
			pmi = self.pmi(words, token)
			#pmi_abstract = self.pmi(abstract, token)
			#pmi_other = self.pmi(other, token)
			#phrases[i]['features'].extend([len_token, tf_idf_abstract, tf_idf_other, first_occur, last_occur, first_occur_abstract, last_occur_abstract, first_occur_other, last_occur_other, pmi, pmi_abstract, pmi_other])
			phrases[i]['features'].extend([len_token, tf_idf_abstract, tf_idf_other, first_occur, last_occur,  pmi])
			
	def count_text_token(self, text, token):
		res = 0
		len_token = len(token)
		for i in range(0, len(text) - len_token + 1):
			if text[i:i+len_token] == token:
				res += 1
		return res
	
	def find_occur(self, text, token):
		first_occur = 0
		last_occur = 0
		len_token = len(token)
		already_find = False
		for i in range(0, len(text) - len_token + 1):
			if text[i:i+len_token] == token:
				if not already_find:
					first_occur = i
					last_occur = i
					already_find = True
				else:
					last_occur = i
		return (float(first_occur) / float(len(text)), float(last_occur) / float(len(text)))
		
	def pmi(self, text, token):
		len_token = len(token)
		tf = float(self.count_text_token(text, token)) / (len(text) - len_token + 1)
		if tf == 0:
			return 0
		pmi = math.log(tf)
		for i in range(0, len(token)):
			pmi -= math.log(float(text.count(token[i])) / len(text))
		pmi = pmi / (math.log(tf) ** (len_token - 1))
		return pmi
	
	def add_learn_matrix(self, phrases): 
		for phrase in phrases:
			self.learn_matrix.append(phrase['features'])
			self.learn_label.append(phrase['label'])
		
	def normalize_matrix(self, matrix):
		res_max = map(abs, matrix[0])
		for item in matrix:
			for i in range(0, len(item)):
				if abs(item[i]) > res_max[i]:
					res_max[i] = abs(item[i])
		for i in range(0, len(matrix)):
			for j in range(0, len(matrix[i])):
				matrix[i][j] = float(matrix[i][j]) / res_max[j]
				
	def learn_model(self):
		numpy.savetxt(learn_matrix_file, self.learn_matrix, delimiter=",", fmt = "%.3f")
		self.normalize_matrix(self.learn_matrix)
		
		#logistic = mlpy.LibLinear(solver_type = 'l1r_lr', C=10, eps=0.0001, weight={0:1, 1:10})
		#logistic.learn(self.learn_matrix, self.learn_label)
		#answers = logistic.pred(self.learn_matrix)
		
		#result = []
		#for i in range(0, len(answers)):
		#	result.append([answers[i], i+1])
		#numpy.savetxt(learn_matrix_file, self.learn_matrix, delimiter=",", fmt = "%.3f")
		#numpy.savetxt(learn_label_file, self.learn_label, delimiter=",", fmt = "%.0f")
		#numpy.savetxt(temp_file, sorted(result), delimiter=",", fmt = "%.1f")
		#print logistic.w()
		
		#if logistic.w() is list:
		#	self.weights = logistic.w()[0]
		#else:
		#	self.weights = logistic.w() 
		#print self.weights
		#self.model = logistic
		
	def save_idf(self):
		file = open(idf_abstract_file, "wb")
		cPickle.dump(self.idf_abstract, file)
		file.close()
		file = open(idf_other_file, "wb")
		cPickle.dump(self.idf_other, file)
		file.close()
		
	def load_idf(self):
		try:
			file = open(idf_abstract_file)
			self.idf_abstract = cPickle.load(file)
			file.close()
			file = open(idf_other_file)
			self.idf_other = cPickle.load(file)
			file.close()
			return True
		except:
			return False
			
	def save_model(self, path_model):
		self.save_idf()
		file = open(file_counter_file, "wb")
		cPickle.dump(self.file_counter, file)
		file.close()
		file = open(weights_file, "wb")
		cPickle.dump(self.model.w(), file)
		file.close()
    
	def load_model(self, path_model):
		try:
			self.load_idf()
			file = open(file_counter_file)
			self.file_counter = cPickle.load(file)
			file.close()
			file = open(weights_file)
			self.weights = cPickle.load(file)
			file.close()
			return True
		except:
			return False
        
	def get_model(self):
		return self.model
		
	def test(self):
		sample_test = os.listdir(test_default_path)
		sample_test = [os.path.join(test_default_path, sample) for sample in sample_test] 
		self.learn(sample_test)
		#if os.path.exists(temp_file):
		#	os.remove(temp_file)
		#temp = open(temp_file, "a")	
		#for (key, item) in sorted(self.idf_abstract.items()):
		#	temp.write(key.encode('utf-8') + ' : ' + str(item) + '\n')
		#temp.write('-----------------------------------------------------------------------------------------\n')
		#for (key, item) in sorted(self.idf_other.items()):
		#	temp.write(key.encode('utf-8') + ' : ' + str(item) + '\n')
		#temp.close()
		#print (self.num_find, self.num_not_find)

class Keywords():
	def __init__(self, load_model = True, save_model = False, sample_learn = None):
		self.learner = Learner()
		if (not load_model):
			if (sample_learn == None):
				self.learner.learn(learn_default_path)
			else: 
				self.learner.learn(sample_learn)
			if save_model:
				self.learner.save_model(load_default_path)
		else:
			self.learner.load_model(pathToModel)
        
	def getKeywords(self, text):
		key_words = self.learner.predict(text)
 		return key_words
				
class Tester():

	def __init__(self, test_path = test_default_path):
		self.test_path = test_path
		
	def write_answers(self, path, answer_list):
		dirname = os.path.dirname(path)
		if not os.path.exists(dirname):
			os.makedirs(dirname)
		file = open(path, "w")
		for line in answer_list:
			file.write(line.encode('utf-8') + '\n')
		file.close()
	
	def run_cv(self, sample_learn_cv, sample_test_cv):
		print "Learning Model..."
		classificator = Keywords(False, False, sample_learn_cv)
		print "Testing Model..."
		i = 1
		for file in sample_test_cv:
			file_test = open(file)
			text = unicode(file_test.read(), "utf-8")
			file_test.close()
			path_res = file.replace('data', 'res')
			print(path_res, i)
			i += 1
			self.write_answers(path_res, classificator.getKeywords(text))
			
	def CV(self, number_blocks  = 5):
		sample_test = os.listdir(self.test_path)
		random.shuffle(sample_test)
		sample_test = [os.path.join(self.test_path, sample) for sample in sample_test] 
		block_size = len(sample_test) / number_blocks
		for q in range(1, number_blocks):
			sample_test_cv = sample_test[((q - 1) * block_size):(q * block_size)]
			sample_learn_cv = list(set(sample_test) - set(sample_test_cv))
			self.run_cv(sample_learn_cv, sample_test_cv) 
		sample_test_cv = sample_test[((number_blocks - 1) * block_size):]
		sample_learn_cv = list(set(sample_test) - set(sample_test_cv))
		self.run_cv(sample_learn_cv, sample_test_cv) 
	
	def stem(self, text):
		stemmer = nltk.stem.snowball.RussianStemmer() 
		text_stem = []
		text = text.split('\n')
		for line in text:
			if line:
				line_free = nltk.PunktWordTokenizer().tokenize(line)
				line_stem = ''
				for word in line_free:
					line_stem += stemmer.stem(word) + ' '
				text_stem.append(line_stem)
		return text_stem
	
	def check_answers(self):
		results = dict()
		results['tp'] = 0
		results['tn'] = 0
		results['fp'] = 0
		results['fn'] = 0
		i = 1
		for file in os.listdir(test_answers_path):
			print (file,i)
			i += 1
			tp = 0
			fp = 0
			fn = 0
			file_test = open(os.path.join(test_answers_path, file))
			text_test = unicode(file_test.read(), "utf-8")
			file_test.close()
			file_right = open(os.path.join(right_answers_path, file))
			text_right = unicode(file_right.read(), "utf-8")
			file_right.close()
			text_test = self.stem(text_test)
			text_right = self.stem(text_right)
			for line in text_test:
				if line in set(text_right):
					results['tp'] += 1
					tp += 1 
				else:
					results['fp'] += 1
					fp += 1
			for line in text_right:
				if not line in set(text_test):
					results['fn'] += 1
					fn += 1
			print "tp = %d, fp = %d, fn = %d" % (tp, fp, fn)		
		results_stat = dict()
		results_stat['precision'] = float(results['tp']) / float(results['tp'] + results['fp']) 
		results_stat['recall'] = float(results['tp']) / float(results['tp'] + results['fn']) 
		results_stat['f-measure'] = 2 * results_stat['precision'] * results_stat['recall'] / (results_stat['precision'] + results_stat['recall'])
		return results_stat

class Runner():
	
	def __init__(self):
		print "Runner - Learn"
		learner = Learner()
		learner.learn([os.path.join(learn_default_path, sample) for sample in os.listdir(learn_default_path)])
		print "Runner - Save"
		learner.save_model(load_default_path)

class ErrorTester():

	def __init__(self):
		print "ErrorTester"
		i = 1
		key_words = Keywords()
		file_tester = open('tester.txt', 'w')
		for file in [os.path.join(test_default_path, sample) for sample in os.listdir(test_default_path)]:
			print (file, i)
			i += 1
			file_test = open(file)
			text_test = unicode(file_test.read(), "utf-8")
			for keyword in key_words.getKeywords(text_test):
				file_tester.write(keyword.encode('utf-8') + '\n')
		file_test.close()
		file_tester.close()
		
class GetStatistics():
	
	def __init__(self):
		self.chunk_size = 100
		self.hist = dict()
		self.hist_logic = dict()
		self.hist_logic[0] = 0
		self.hist_logic[1] = 0
		self.len_hist = dict()
		self.len_token_hist = dict()
		self.log = dict()
		self.log_exist = dict()
		self.log_exist[0] = 0
		self.log_exist[1] = 0
		
		stemmer = nltk.stem.snowball.RussianStemmer()
		num = 1
		numi = 0
		for filename in os.listdir(learn_default_path):
			print(filename, num)
			num += 1
			file_test = open(os.path.join(learn_default_path, filename))
			text_test = unicode(file_test.read(), "utf-8")
			#text_test = re.sub('\d', '', text_test)
			file_test.close()
			file_right = open(os.path.join(right_answers_path, filename))
			text_right = unicode(file_right.read(), "utf-8")
			file_right.close()
			
			text_test = nltk.PunktWordTokenizer().tokenize(text_test)
			text_test = map(stemmer.stem, text_test)
			text_test_split = self.get_logical_struct(text_test)
			if len(text_test_split) == 1:
				self.log_exist[0] += 1
			else:
				self.log_exist[1] += 1
				self.log[num] = len(text_test_split[0]) 
			#temp = open(another_temp_file, "w")
			#for i in range(0, len(text_test_split)):
			#	for j in range(0, len(text_test_split[i])):
			#		temp.write(text_test_split[i][j].encode('utf-8') + '\n')
			#	temp.write('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n')
			#temp.close()
			#break
			text_right = self.stem(text_right)
			if u'' in text_test:
				numi += 1
			for token in text_right:
				res_token = self.find_token(text_test, token)
				for value in res_token:
					if not value in self.hist.keys():
						self.hist[value] = 1
					else:
						self.hist[value] += 1
				if self.find_token_logical(text_test_split[0], token):
					self.hist_logic[0] += 1
				else: 
					self.hist_logic[1] += 1
		print '======================================================================================'
		print numi
		print '======================================================================================'
		
	def stem(self, text):
		stemmer = nltk.stem.snowball.RussianStemmer() 
		text_stem = []
		text = text.split('\n')
		for line in text:
			if line:
				line_free = nltk.PunktWordTokenizer().tokenize(line)
				line_stem = []
				for word in line_free:
					line_stem.append(stemmer.stem(word))
				text_stem.append(line_stem)
		return text_stem 
		
	def find_token(self, text, token):
		len_token = len(token)
		if not len_token in self.len_token_hist.keys():
			self.len_token_hist[len_token] = 1
		else:
			self.len_token_hist[len_token] += 1
		for i in range(0, len_token):
			if not len(token[i]) in self.len_hist.keys():
				self.len_hist[len(token[i])] = 1
			else:
				self.len_hist[len(token[i])] += 1
		
		temp = open(temp_file, "a")	
		for i in range(0, len_token):
			temp.write(token[i].encode('utf-8') + ' ')
		temp.write('\n')
		for i in range(0, 50):
			temp.write(text[i].encode('utf-8') + '\n')
		temp.close()
		res = list()
		for i in range(0, len(text) - len_token + 1):
			if text[i:i+len_token] == token:
				res.append(i / self.chunk_size)
		return res
	
	def find_token_logical(self, text, token):
		len_token = len(token)
		for i in range(0, len(text) - len_token + 1):
			if text[i:i+len_token] == token:
				return True
		return False
		
	def write_hist(self):
		print "Distribution of words over chunks"
		for key in sorted(self.hist):
			print "%d : %d" % (key, self.hist[key])
		print '----------------------------------------------------------'
		print "Distribution of keyphrases length"
		for key in sorted(self.len_token_hist):
			print "%d : %d" % (key, self.len_token_hist[key])
		print '----------------------------------------------------------'
		print "Distribution of keywords length"
		for key in sorted(self.len_hist):
			print "%d : %d" % (key, self.len_hist[key])
		print '----------------------------------------------------------'
		print sorted(self.log.values())[len(self.log.values()) / 2]
		print '----------------------------------------------------------'
		print "Distribution of abstract exist"
		for key in sorted(self.log_exist):
			print "%d : %d" % (key, self.log_exist[key])
		print '----------------------------------------------------------'
		print "Distribution of keywords exist in abstract"
		for key in sorted(self.hist_logic):
			print "%d : %d" % (key, self.hist_logic[key])
		print 'Recall = %f' % (float(self.hist_logic[0]) / float(self.hist_logic[0] + self.hist_logic[1]))
		print '----------------------------------------------------------'
	
	def get_logical_struct(self, stemmed_text):
		key_word = u''
		for i in range(0, len(stemmed_text)):
			if stemmed_text[i] == key_word:
				res = list()
				if i > 300:
					res.append(stemmed_text[0:default_abstract_length])
					res.append(stemmed_text[default_abstract_length:])
				else:
					res.append(stemmed_text[0:i])
					res.append(stemmed_text[i:])
				return res
		res = list()
		res.append(stemmed_text[:default_abstract_length])
		res.append(stemmed_text[default_abstract_length:])
		return res
		
#learner = Learner()
#learner.test()
#text_processor = TextProcessor()
#text_processor.test()
#stat = GetStatistics()
#stat.write_hist()			
#if __name__ == '__main__':
#	tester = Tester()
#	tester.CV(3)
#	print tester.check_answers()
#runner = Runner()
#ErrorTester()
